 /*
 * ------------------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 23):
 * As long as you retain this notice you can do whatever you want with this stuff.
 * If we meet some day, and you think this stuff is worth it, you can buy me a beer
 * in return. Niklas Roy.
 * Revision 23 is based on revision 42 of this license by Poul-Henning Kamp
 *
 *       +-------------------------------------+
 *       |        MICRO MONSTER MODULAR        |
 *       |          -  - ------- -  -          |
 *       |   VARIATIONS ON t*(t>>9|t>>13)&16   |
 *       +-------------------------------------+
 *
 * Code is written in Arduino-style C.
 * Program uses direct access to ATmega8 registers for wave sample output (PORTB=...)
 * Thus only works with an ATmega8 with external 16MHz crystal!
 * (Well, did not test this, but might work on an Atmega168 as well.)
 * ------------------------------------------------------------------------------------
 */

 //--------------------------------------------------- IO variables:
byte button1,button2;                               // buttons
byte poti1;                                         // poti left1
byte poti2;                                         // poti left2
byte poti3;                                         // poti left3
byte poti4;                                         // poti left4
byte count;                                         // counter for reading potis

 //--------------------------------------------------- wave sample calculation variables:                      
long t;                                             // counter for wave sample calculation
unsigned int wave;                                  // stores latest wave sample
long wave_sum;                                      // stores added sample values for low pass filtering

 //--------------------------------------------------- low pass filter variables:
byte ring_count;                                    // ring buffer index for low pass filter, rolls over fro 255 to 0
byte ring[256];                                     // ring buffer stores old wave sample values for low pass filter

void setup() {                
 //--------------------------------------------------- IO port settings :
  DDRB= 0b00111111;                                 // 6bit DAC (audio out) on PORTB 0...5
  PORTD=0b00001100;                                 // buttons are connected to d2 and d3 - internal pull-ups are switched on

  for (unsigned int s=8192;s<16384;s++){            // little startup sound
    PORTB=(s*(s>>9|s>>13)&32)+((s-2)*((s-2)>>9|(s-2)>>13)&32);
    delayMicroseconds(128);
  }
}

void loop() {
 //--------------------------------------------------- read potis:
 count++;                                           // byte will roll over from 255 to 0
                                                    // potis are read only every 64th cycle
                                                    // one poti after another
                                                    // some potis are reversed (255-...)
 if (count==0)   poti1 = 255-(analogRead(0)>>2);    // poti left1
 if (count==64)  poti2 = 255-(analogRead(1)>>2);    // poti left2
 if (count==128) poti3 =      analogRead(2)>>2;     // poti left3
 if (count==192) poti4 = 255-(analogRead(3)>>2);    // poti left4 
 
 //--------------------------------------------------- read buttons:
 button1=!digitalRead(3);
 button2=!digitalRead(2);
 
 if   (button2||button1){                           // only if at least one of the buttons is pushed:
   if (button1)t+=16;                               // increment t more or less, 
   if (button2)t++;                                 // based on which buttons are pressed
   
 //--------------------------------------------------- calculate wave sample :
 //                                                    calculation is based on  t*(t>>9|t>>13)&16 
 
 //      | a  |        b        |         c          |        d        |
   wave = t*  (t >> (poti1>>4)  |  t >> (poti2>>4) ) & ( (poti3>>3) +16) ;
   
 //  a: binary count (increments more or less depending on which button is pushed)
 //  b: bit pattern is shifted based on poti 1     (will cause tonal shifts by octaves)
 //  b: bit pattern is shifted based on poti 2     (will cause tonal shifts by octaves)
 //  c: bit mask cancels out bits based on poti 3  (will filter overtones)
     
 //--------------------------------------------------- low pass filter :
   ring_count++;                                    // increment ring buffer index
   ring[ring_count]=wave;                           // store actual wave sample in ring buffer
   
   wave_sum=0;                                      // reset wave_sum
   byte filter=poti4>>2;                            // filter value is poti4/4
   if (!filter) filter=1;                           // filter cannot be zero
   for (byte i=filter;i>0;i--){                     // loop accesses past wave samples in ring buffer
     wave_sum+=ring [byte(ring_count-i)];           // wave_sum adds up past wave sample values
   }
   wave_sum/=filter;                                // sum of wave samples is divided by filter value (averaging between all added values)
   
 //--------------------------------------------------- audio output :
   PORTB=wave_sum;                                  // PORTB 0...5 is directly connected to DAC 
                                                    // thus more significant bits are automatically discarded.
                                                    
                                                    
 }else{                                             // if no button is pushed:
   PORTB=0;                                         // set DAC to 0V
 }
}











