/*
  micro monster modular
  testprogram
  
  - button 1 will play a wobbled sine wave. sine wave frequency is based on poti1. wobble frequency is based on poti2.
  - button2 will play a square wave. square wave frequency is based on poti3. square wave amplitude is based on poti4.
  - wobbled sine wave and square wave are added. this can cause variable roll overs (which will appear as noise) when poti4 is positioned in the upper half. 
  
  beer ware license rev. 23
  n. roy
  2013
 */
 
#define SOUNDOUT PORTB                                           // 6-bit DAC for sound output is connected to PORTB. The two MSB will not be regarded due to the DDRB setting ...
                                                                 // ...which means, SOUNDOUT can be set to any value - while only the 6 lowest bits will have an effect.
byte poti1;                                                      // poti left1
byte poti2;                                                      // poti left2
byte poti3;                                                      // poti left3
byte poti4;                                                      // poti left4
byte adc_count;                                                  // counter variable used to slow down adc readouts

unsigned int sin_count,wobble_count,square_count;
byte wave_out;

byte sine[256]={                                                 // look up table of a sinewave will be handy later:
0, 1, 1, 1, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 16, 17, 19, 21, 22, 24, 26, 28, 30, 32, 34, 36, 38, 41, 43, 45, 48, 50, 
53, 55, 58, 60, 63, 66, 69, 71, 74, 77, 80, 83, 86, 89, 92, 95, 98, 101, 104, 107, 110, 114, 117, 120, 123, 126, 128, 131, 135, 138, 
141, 144, 147, 150, 153, 156, 159, 163, 166, 169, 172, 174, 177, 180, 183, 186, 189, 192, 194, 197, 200, 202, 205, 207, 210, 212, 214, 
217, 219, 221, 223, 225, 227, 229, 231, 233, 235, 236, 238, 240, 241, 242, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 253, 254, 
254, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 254, 254, 253, 253, 252, 251, 250, 249, 248, 247, 246, 245, 244, 242, 241, 240, 
238, 236, 235, 233, 231, 229, 227, 225, 223, 221, 219, 217, 214, 212, 210, 207, 205, 202, 200, 197, 194, 192, 189, 186, 183, 180, 177, 
174, 172, 169, 166, 163, 159, 156, 153, 150, 147, 144, 141, 138, 135, 131, 128, 126, 123, 120, 117, 114, 110, 107, 104, 101, 98, 95, 
92, 89, 86, 83, 80, 77, 74, 71, 69, 66, 63, 60, 58, 55, 53, 50, 48, 45, 43, 41, 38, 36, 34, 32, 30, 28, 26, 24, 22, 21, 19, 17, 16, 
14, 13, 12, 10, 9, 8, 7, 6, 5, 4, 4, 3, 2, 2, 1, 1, 1, 1, 1, 0 };


void setup() {                
                                                                 // set pin drive directions and internal pull-up's:
  PORTC=0b01000000;                                              // internal pull-up resistor for reset pin is switched on
  DDRD= 0b00000000;                                              // all portd is in, buttons on d2 and d3
  PORTD=0b00001100;                                              // internal pull-ups for button pins are switched on
  DDRC= 0b00000000;                                              // all portc is analog input
  DDRB= 0b00111111;                                              // 6bit digital out on portb
                                       
  for (byte i=0;i<255;i++){                                      // little startup sound 
    for (int j=0;j<1024;j++){
      sin_count+=sine[i];
      SOUNDOUT=sine[sin_count>>8]>>2;
      delayMicroseconds(1);
    }
  }            
  
  SOUNDOUT=0;                                                    // set the DAC to 0V
}

void loop() {
     adc_count++;                                                // increment adc_count (it's a byte variable, so it will roll over when it reaches 256)
     switch (adc_count){                                         // trigger the adc reads - they shall only happen each 64th loop cycle, because i want things to happen fast
       case 0:   poti1 = analogRead(0)>>2; break;                // read poti #1 (which is the most outside one)                  +--------------------------------------------------------------+
       case 63:  poti2 = analogRead(1)>>2; break;                // read poti #2                                                  | analogRead() returns a 10-bit value (0-1023).                |
       case 127: poti3 = analogRead(2)>>2; break;                // read poti #3                                                  | i'm only interested in the upper 8 bits                      |
       case 191: poti4 = analogRead(3)>>2; break;                // read poti #4 (that's the one which is close to the speaker)   | so the >> operator is used to shifts all bits two positions  | 
     }                                                           //                                                               | to the right, then they fit in a byte and have a range       |
                                                                 //                                                               | from 0-255. (>>2 computes faster than the /4 operation.)     |
                                                                 //                                                               +-------------------------------------------------------------+
   byte button1=!digitalRead(3);                                 // read the upper button 
   byte button2=!digitalRead(2);                                 // read the lower button
   
   wave_out=0;                                                   // reset the value which will later go out to the DAC
   
   if(button1){                                                  // is button 1 pushed?
    wobble_count+=(poti2>>2);                                    // increment wobble counter by poti2/4          
    sin_count+=( poti1<<2)+sine[wobble_count>>8];                // increment the sinewave counter by (poti1*4) and by the wobble
    wave_out=sine[sin_count>>8]>>3;                              // calculate the sinewave which goes out to the DAC; the output level will range from 0 to 32 (5 bits) 
   }else{
    wobble_count=0;                                              // reset counters if button1 is not pushed
    sin_count=0;
   }
  
   if (button2){                                                 // is button 2 pushed?
     square_count+=poti3<<2;                                     // generate a square wave; frequency is determined by poti3
     if (square_count>>15)wave_out+=poti4>>2;                    // add square wave (amplitude is defined by poti4) to sinewave of button1. values can exceed 6bits, this will cause noise when both buttons are pushed and poti4 value is > 128 
   }
   SOUNDOUT=wave_out%64;                                         // send value to DAC
}






